@@ -860,7 +860,7 @@ configinfo::configinfo(parsestream &i){
   linenumber=1;
   treew="c(m)";
   userpref=rootpref=sort=prerun=preruntest=postrun=genmenu=
-    hkexclude=startmenu=endmenu=submenutitle=NULL;
+    hkexclude=startmenu=endmenu=submenutitle=also_run=NULL;
   mainmt="Debian";
   onlyuniquetitles=onlyrunasroot=onlyrunasuser=false;
   repeat_lang=NULL;
@@ -910,11 +910,13 @@ configinfo::configinfo(parsestream &i){
 	  hkexclude=get_eq_cat_str(i);
 	else if(name==String("genmenu"))
 	  genmenu=get_eq_cat_str(i);	
-	else if(name==String("postrun"))
-	  postrun=get_eq_cat_str(i);	
 	else if(name==String("prerun"))
 	  prerun=get_eq_cat_str(i);	
-	else if(name==String("preruntest"))
+	else if(name==String("postrun"))
+	  postrun=get_eq_cat_str(i);
+	else if(name==String("also_run"))
+	  also_run=get_eq_cat_str(i);
+       	else if(name==String("preruntest"))
 	  preruntest=get_eq_cat_str(i);	
 	else if(name==String("onlyrunasroot"))
 	  onlyrunasroot=i.get_eq_boolean();	
@@ -1128,7 +1130,8 @@ void read_input(parsestream &i){
       check_vars(i,m);
 
       break_slashes(m[SECTION_VAR],sec_vec);
-      sec_vec.push_back(m[TITLE_VAR]);	  
+      if(m[TITLE_VAR] != "/")
+	sec_vec.push_back(m[TITLE_VAR]);	  
       if(supported->prec(m[NEEDS_VAR])!=MAXINT)
 	menu.add_entry(sec_vec,m);
       i.skip_line();   //read away the final newline
@@ -1211,7 +1214,7 @@ int main(int argc, char **argv){
   parsestream *ps, *psscript;
   String menudefname;
   
-  setlocale (LC_ALL, "");
+  setlocale (LC_MESSAGES, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
@@ -1281,8 +1284,29 @@ int main(int argc, char **argv){
     };
     menu.postprocess(1,0, config->rootsection());
     menu.output();
-    
     closegenoutput();
+
+    if(config->also_run){
+      String run=config->also_run->soutput(menu.vars);
+      StrVec run_vec;
+      StrVec::iterator run_i;
+
+      break_char(run, run_vec, ':');
+
+      configinfo *old_config=config;  /* save old config data */
+      parsestream *also_ps;
+
+      for(run_i=run_vec.begin(); run_i!=run_vec.end(); run_i++){
+	cout<<"Running :\""<<*run_i<<"\""<<endl;
+	also_ps=new parsestream(*run_i);
+	config=new configinfo(*also_ps);
+	menu.output();
+	closegenoutput();
+	delete config;
+	delete also_ps;
+      }
+      config=old_config;             /* restore old config data */
+    }
     if(config->rcfile().length()&&config->examplercfile().length())
       includemenus("include-menu-defs",
 		   config->prefix()+"/"+config->genmenu->soutput(menu.vars));
