@@ -12,6 +12,10 @@ class menuentry;
 
 bool operator< (const StrVec &left, const StrVec &right);
 
+/*in submenu_container:
+  StrVec    first   - full title, broken into a StrVec
+  menuentry *second - the menu entry with this title
+*/
 typedef map <StrVec, menuentry *, less<StrVec> > submenu_container;
 
 class menuentry {
@@ -32,6 +36,7 @@ public:
   void output();
   void postprocess(int n_parent, int level, String title);
   void process_hints(const StrVec &pref);
+  void debug(int);
   menuentry(){
     forced=false;
   }
