@@ -15,6 +15,7 @@
 #include <sys/stat.h>
 
 #include "menu-method.h"
+#include "menu-tree.h"
 
 int show_time=0, verbose=0, dodebug=0;
 
@@ -25,7 +26,9 @@ configinfo *config;
 supportedinfo *supported;
 int linenumber=-123456;
 
-ofstream *genoutputfile;
+ofstream *genoutputfile=NULL;
+set<String> outputnames; //all names of files ever written to
+
 
 struct option long_options[] = { 
   { "showtime", no_argument, &show_time, 1 }, 
@@ -78,11 +81,25 @@ void add_functions(){
   store_func(new tolower_func);  
   store_func(new toupper_func);
   store_func(new replacewith_func);
+  store_func(new nstring_func);  
   store_func(new cppesc_func);
   store_func(new parent_func);
   store_func(new basename_func);
   store_func(new entrycount_func);
   store_func(new entryindex_func);
+  store_func(new firstentry_func);
+  store_func(new lastentry_func);
+  store_func(new level_func);
+
+  store_func(new rcfile_func);
+  store_func(new examplercfile_func);
+  store_func(new mainmenutitle_func);
+  store_func(new rootsection_func);
+  store_func(new rootprefix_func);
+  store_func(new userprefix_func);
+  store_func(new treewalk_func);
+  store_func(new postoutput_func);
+  store_func(new preoutput_func);
 }
 
 
@@ -102,25 +119,25 @@ cat_str *get_eq_cat_str(parsestream &i){
 
 int check_dir(String s){
   String t;
-  int i;
+  unsigned int i;
   if(dodebug)
     cerr<<"CHECK_DIR: "<<s<<endl;
   while(s.length()){
-    i=s.index('/',1);
-    if(i>=0){
-      t=s.before('/', 1)+String('/');
+    i=s.find('/',1);
+    if(i<s.length()){
+      t=s.substr(0,i+1);
       for(; ((unsigned int)i<s.length()) && (s[i]=='/'); i++);
-      s=s.after(i-1);
+      s=s.substr(s.begin()+i,s.end());
     } else {
       t=s;
       s="";
     }
-    if(chdir(t)<0){
+    if(chdir(t.c_str())<0){
       if(dodebug)
 	cerr<<"MKDIR "<<t<<endl;
-      if(mkdir(t,0755))
+      if(mkdir(t.c_str(),0755))
 	throw dir_createerror(t);
-      if(chdir(t))
+      if(chdir(t.c_str()))
 	throw dir_createerror(t);
     } else {
       if(dodebug)
@@ -131,28 +148,46 @@ int check_dir(String s){
 }
 
 void closegenoutput(){
-  (*genoutputfile)<<config->postoutput();
-  delete genoutputfile;
+  set<String>::iterator i;
+
+  // OK, this will look clumsy in strace output, especially if there's
+  // only output file: first we close, and immediately afterwards we
+  // open and write again. But I don't see an easy and general way
+  // to get round that.
+  if(genoutputfile){
+    delete genoutputfile;
+    genoutputfile=NULL;
+  }
+  for(i=outputnames.begin(); i!=outputnames.end(); i++){
+    ofstream f((*i).c_str(), ios::app);
+    f<<(config->postoutput());
+  }
 }
 void genoutput(const String &s,
 	       map<String, String, less<String> > &v){
   
-  static String outputname="";
-  const char *t;
+  static String lastname="////";
   String name;
   String dir;
 
   name=config->prefix()+"/"+config->genmenu->soutput(v);
-  if(dodebug)
-    cerr<<"GENOUTPUT: name="<<name<<endl;
-  if(!(name==outputname)){
+  //if(dodebug)
+  //  cerr<<"GENOUTPUT: name="<<name<<endl;
+  if(name!=lastname){
     if(genoutputfile)
-      closegenoutput();
-    outputname=name;
-    check_dir(String_parent(outputname));
-    t=outputname;
-    genoutputfile=new ofstream(t);
-    (*genoutputfile)<<config->preoutput();    
+      delete genoutputfile;
+    if(outputnames.find(name)==outputnames.end()){
+      outputnames.insert(name);
+      check_dir(String_parent(name));
+      // after opening a file with ios::trunc, all writes seem to fail!
+      //genoutputfile=new ofstream(name.c_str(), ios::trunc);
+      // So, I do it this way instead:
+      unlink(name.c_str());
+      genoutputfile=new ofstream(name.c_str());
+      (*genoutputfile)<<config->preoutput();    
+    } else
+      genoutputfile=new ofstream(name.c_str(),ios::app);      
+    lastname=name;
   }
   (*genoutputfile)<<s;
 }
@@ -235,6 +270,7 @@ func_str::func_str(parsestream &i){
 
 ostream &const_str::output(ostream &o, 
 			   map<String, String, less<String> > &/*menuentry*/){
+  //  cout<<"IN CONST_STR: data="<<data<<endl;
   return o<<data;
 }
 
@@ -243,8 +279,16 @@ String cat_str::soutput(map<String, String, less<String> > &menuentry){
   vector <str * >::iterator i;
   ostrstream s(buf,sizeof(buf));
   
-  for(i=v.begin();i!=v.end();i++)
+  //cout<<"CONFIG->rfc="<<config->rcff()<<endl;
+  
+  for(i=v.begin();i!=v.end();i++){
+    //char Duf[MAX_BUF];
+    //ostrstream D(Duf,sizeof(buf));
     (*i)->output(s,menuentry);
+    //(*i)->output(D,menuentry);
+    //D<<ends;
+    //cout<<"DDD "<<Duf<<endl;
+  }
   s<<ends;
   
   return String(buf);
@@ -346,7 +390,7 @@ ostream &iffile_func::output(ostream &o, vector<cat_str *> &args,
 			     map<String, String, less<String> > &menuentry){
 
   String s=args[0]->soutput(menuentry);
-  ifstream f(s);
+  ifstream f(s.c_str());
   if(f)
     args[1]->output(o,menuentry);
   return o;
@@ -355,7 +399,7 @@ ostream &ifelsefile_func::output(ostream &o, vector<cat_str *> &args,
 			     map<String, String, less<String> > &menuentry){
 
   String s=args[0]->soutput(menuentry);
-  ifstream f(s);
+  ifstream f(s.c_str());
   if(f)
     args[1]->output(o,menuentry);
   else
@@ -366,7 +410,7 @@ ostream &catfile_func::output(ostream &o, vector<cat_str *> &args,
 			     map<String, String, less<String> > &menuentry){
 
   String s=args[0]->soutput(menuentry);
-  ifstream f(s);
+  ifstream f(s.c_str());
   char c;
 
   while(f && f.get(c))
@@ -397,9 +441,9 @@ ostream &escfirst_func::output(ostream &o, vector<cat_str *> &args,
 
   for(i=0;(unsigned int)i!=s.length();i++){
     if(esc.length() && esc.contains(s[i])){
-      t=s.before(i);
+      t=s.substr(0,i);
       t+=args[2]->soutput(menuentry);
-      t+=s.after(i-1);
+      t+=s.substr(s.begin()+i,s.end());
       break;
     }
     t+=s[i];
@@ -423,6 +467,16 @@ ostream &replacewith_func::output(ostream &o, vector<cat_str *> &args,
 			       args[1]->soutput(menuentry),
 			       args[2]->soutput(menuentry));
 }
+ostream &nstring_func::output(ostream &o, vector<cat_str *> &args,
+			    map<String, String, less<String> > &menuentry){
+  int count= Stringtoi(args[0]->soutput(menuentry));
+  int i;
+
+  for(i=0; i<count; i++)
+    o<<args[1]->soutput(menuentry);
+
+  return o;
+}
 ostream &cppesc_func::output(ostream &o, vector<cat_str *> &args,
 			      map<String, String, less<String> > &menuentry){
 
@@ -534,18 +588,8 @@ ostream &basename_func::output(ostream &o, vector<cat_str *> &args,
 			       map<String, String, less<String> > &menuentry){
 
   String s=args[0]->soutput(menuentry);
-  String t;
-  int  i,p,q;
   
-  for(i=0,p=-1,q=0;(unsigned int)i!=s.length();i++)
-    if(s[i]=='/'){
-      q=p;
-      p=i;
-    }
-  if(p<0)
-    return o;
-  else
-    return o<<s.at(q+1,p-q-1);
+  return o<<String_basename(s);
 }
 ostream &entrycount_func::output(ostream &o, vector<cat_str *> &/*args*/,
 				 map<String, String, less<String> > &menuentry){
@@ -555,6 +599,70 @@ ostream &entryindex_func::output(ostream &o, vector<cat_str *> &/*args*/,
 				 map<String, String, less<String> > &menuentry){
   return o<<menuentry[PRIVATE_ENTRYINDEX_VAR];
 }
+ostream &firstentry_func::output(ostream &o, vector<cat_str *> &args,
+				 map<String, String, less<String> > &menuentry){
+  int index=Stringtoi(menuentry[PRIVATE_ENTRYINDEX_VAR]);
+
+  if(index == 0)
+    args[0]->output(o,menuentry);
+  return o;
+}
+
+ostream &lastentry_func::output(ostream &o, vector<cat_str *> &args,
+				 map<String, String, less<String> > &menuentry){
+
+  int index=Stringtoi(menuentry[PRIVATE_ENTRYINDEX_VAR]);
+  int count=Stringtoi(menuentry[PRIVATE_ENTRYCOUNT_VAR]);
+
+  if(index+1 == count)
+    args[0]->output(o,menuentry);
+  return o;
+}
+
+
+ostream &level_func::output(ostream &o, vector<cat_str *> &/*args*/,
+				 map<String, String, less<String> > &menuentry){
+  return o<<menuentry[PRIVATE_LEVEL_VAR];
+}
+
+
+ostream &rcfile_func::output(ostream &o, vector<cat_str *> &/*args*/,
+				 map<String, String, less<String> > &menuentry){
+  return o<<config->rcfile();
+}
+ostream &examplercfile_func::output(ostream &o, vector<cat_str *> &/*args*/,
+				 map<String, String, less<String> > &menuentry){
+  return o<<config->examplercfile();
+}
+ostream &mainmenutitle_func::output(ostream &o, vector<cat_str *> &/*args*/,
+				 map<String, String, less<String> > &menuentry){
+  return o<<config->mainmenutitle();
+}
+ostream &rootsection_func::output(ostream &o, vector<cat_str *> &/*args*/,
+				 map<String, String, less<String> > &menuentry){
+  return o<<config->rootsection();
+}
+ostream &rootprefix_func::output(ostream &o, vector<cat_str *> &/*args*/,
+				 map<String, String, less<String> > &menuentry){
+  return o<<config->rootprefix();
+}
+ostream &userprefix_func::output(ostream &o, vector<cat_str *> &/*args*/,
+				 map<String, String, less<String> > &menuentry){
+  return o<<config->userprefix();
+}
+ostream &treewalk_func::output(ostream &o, vector<cat_str *> &/*args*/,
+				 map<String, String, less<String> > &menuentry){
+  return o<<config->treewalk();
+}
+ostream &postoutput_func::output(ostream &o, vector<cat_str *> &/*args*/,
+				 map<String, String, less<String> > &menuentry){
+  return o<<config->postoutput();
+}
+ostream &preoutput_func::output(ostream &o, vector<cat_str *> &/*args*/,
+				map<String, String, less<String> > &menuentry){
+  return o<<config->preoutput();
+}
+
 
 /////////////////////////////////////////////////////
 //  "defined" function (macro).
@@ -648,8 +756,8 @@ int supportedinfo::prec(String &name){
     p=MAXINT;
   else
     p=(*i).second.prec;
-  if(dodebug)
-    cerr<<"PREC: name="<<upcase(name)<<"="<<p<<endl;
+  //if(dodebug)
+  //  cerr<<"PREC: name="<<upcase(name)<<"="<<p<<endl;
   return p;
 }
 
@@ -663,14 +771,67 @@ ostream &supportedinfo::debuginfo(ostream &o){
   return o;
 }
 /////////////////////////////////////////////////////
+//  forcetree stuff
+//
+
+void read_forcetree(parsestream &i){
+  Regex r("[a-zA-Z/-_ ]");
+
+  while(1){
+    try{
+      String name;
+      StrVec v;
+      menuentry *m;
+
+      i.skip_space();
+      name=i.get_name(r);
+      if(!name.size())
+	throw ident_expected(&i);
+      if(name==String("endforcetree"))
+	return;
+      
+      break_slashes(name,v);
+      m=new menuentry;
+      m->forced=true;
+      m->vars[TITLE_VAR]=v[v.size()-1];
+      menu.add_menuentry_ptr(v,m);  
+      i.skip_line();
+    }
+    catch(endofline d){}
+  }
+  
+}
+/////////////////////////////////////////////////////
 //  configinfo
 //
 configinfo::configinfo(parsestream &i){
   String errmsg("Parse error: String constant expected");
   linenumber=1;
   treew="c(m)";
-  prerun=postrun=genmenu=hkexclude=startmenu=endmenu=submenutitle=NULL;
+  sort=prerun=preruntest=postrun=genmenu=
+    hkexclude=startmenu=endmenu=submenutitle=NULL;
+  mainmt="Debian";
+  onlyrunasroot=onlyrunasuser=false;
+
+  hint_optimize=false;
+  hint_nentry=6;
+  hint_topnentry=5;
+  hint_mixedpenalty=15;
+  hint_minhintfreq=0.1;
+  hint_mlpenalty=2000;
+  hint_max_ntry=4;
+  hint_max_iter_hint=5;
+  hint_debug=false;
+
+  //minhintfreq=0.1;
+  //hintdebug=false;
+  //n_entry_opt=6;
+  //hint_toplevel_nopt=5;
+  //hint_mlpenalty=20;
+  //keep_sections=true;
+
   preout="#Automatically generated file. Do not edit (see /usr/doc/menu/html/index.html)\n\n";
+  postout="";
   roots="/Debian";
   try{
     while(1){
@@ -679,7 +840,10 @@ configinfo::configinfo(parsestream &i){
 	if(name==String("supported")){
 	  i.skip_line();
 	  supported=new supportedinfo(i);
-	}
+	} else if(name==String("forcetree")){
+	  i.skip_line();
+	  read_forcetree(i);
+	} 
 	else if(name==String("function"))
 	  store_func(new func_def(i));
 	else if(name==String("startmenu"))
@@ -696,12 +860,23 @@ configinfo::configinfo(parsestream &i){
 	  postrun=get_eq_cat_str(i);	
 	else if(name==String("prerun"))
 	  prerun=get_eq_cat_str(i);	
+	else if(name==String("preruntest"))
+	  preruntest=get_eq_cat_str(i);	
+	else if(name==String("onlyrunasroot"))
+	  onlyrunasroot=i.get_eq_boolean();	
+	else if(name==String("onlyrunasuser"))
+	  onlyrunasuser=i.get_eq_boolean();	
+	else if(name==String("sort"))
+	  sort=get_eq_cat_str(i);
 
 	else if(name==String("compat")){
-	  if((compt=i.get_eq_Stringconst())!=String("menu-1")){
-	    cerr<<"Unknown compat mode "<<compt<<" in config file (only know of \""COMPAT_MODE"\"";
-	    throw informed_fatal();
-	  }
+	  compt=i.get_eq_Stringconst();
+	  if(compt==String("menu-1"))
+	    i.seteolmode(parsestream::eol_newline);
+	  else if(compt==String("menu-2"))
+	    i.seteolmode(parsestream::eol_semicolon);
+	  else 
+	    throw unknown_compat(&i, compt);
 	}
 	else if(name==String("rcfile"))
 	  rcf=i.get_eq_Stringconst();
@@ -722,7 +897,7 @@ configinfo::configinfo(parsestream &i){
 	else if(name==String("preoutput"))
 	  preout=i.get_eq_Stringconst();
 	else if(name==String("command")){
-	  system(i.get_eq_Stringconst());
+	  system((i.get_eq_Stringconst()).c_str());
 	  exit(0);
 	}
 	else if(name==String("hotkeycase")){
@@ -735,7 +910,28 @@ configinfo::configinfo(parsestream &i){
 	    cerr<<"install-menus hotkeycase can only be {,in}sensitive"<<endl;
 	    throw informed_fatal();
 	  }
+	} 
+	else if(name==String("hint_optimize"))
+	  hint_optimize=i.get_eq_boolean();
+	else if(name==String("hint_nentry"))
+	  hint_nentry=i.get_eq_double();
+	else if(name==String("hint_topnentry"))
+	  hint_topnentry=i.get_eq_integer();
+	else if(name==String("hint_mixedpenalty"))
+	  hint_mixedpenalty=i.get_eq_double();
+	else if(name==String("hint_minhintfreq"))
+	  hint_minhintfreq=i.get_eq_double();
+	else if(name==String("hint_mlpenalty"))
+	  hint_mlpenalty=i.get_eq_double();
+	else if(name==String("hint_max_ntry")){
+	  hint_max_ntry=i.get_eq_integer();
+	  if(hint_max_ntry < 1)
+	    hint_max_ntry=1;
 	}
+	else if(name==String("hint_max_iter_hint"))
+	  hint_max_iter_hint=i.get_eq_integer();
+	else if(name==String("hint_debug"))
+	  hint_debug=i.get_eq_boolean();
 	else
 	  throw unknown_ident(&i);
       i.skip_line();//read away final newline
@@ -754,18 +950,10 @@ void configinfo::check_config(){
   }
 }
 String configinfo::prefix(){
-  DIR *d;
   String s;
 
-  if(getuid()){
-    s=String(getenv("HOME"))+"/"+userprefix();
-    d=opendir(s);
-    if(d){
-      closedir(d);
-      return s;
-    } else
-      return rootpref;
-  }
+  if(getuid())
+    return String(getenv("HOME"))+"/"+userprefix();
   else
     return rootpref;
 }
@@ -795,166 +983,6 @@ ostream &configinfo::debuginfo(ostream &o){
   return o;
 }
 
-/////////////////////////////////////////////////////
-//  Menuentry
-//
-
-void menuentry::add_entry(String sec, String fullsec, 
-			  map<String, String, less<String> > &v){
-  if(dodebug)
-    cerr<<"ADDENTRY: sec="<<sec<<", needs="<<v["needs"]<<endl;
-  if(sec.length()!=0){
-    String name;
-    map <String, menuentry *, less<String> >::iterator i;
-    if(sec.contains("/")){
-      name=sec.before("/");
-      sec=sec.after("/");
-    }
-    else{
-      name=sec;
-      sec="";
-      if(!v[COMMAND_VAR].length()){
-	vars[TITLE_VAR]=name;
-	vars=v;
-	vars[SECTION_VAR]=fullsec;
-	return;
-      }
-    }
-    if(dodebug)
-      cerr<<"ADD_ENTRY: name="<<name<<", sec="<<sec<<endl;
-    if((i=submenus.find(name))!=submenus.end()){
-      if (!sec.length()){
-	if((supported->prec(((*i).second)->vars[NEEDS_VAR]) >
-	    supported->prec(v[NEEDS_VAR]))){
-	  delete submenus[name];
-	  submenus[name]=new menuentry;
-	} else
-	  return;
-      }
-    }else
-      submenus[name]=new menuentry;
-    submenus[name]->add_entry(sec, fullsec+"/"+name , v);
-    if(!submenus[name]->vars[TITLE_VAR].length())
-      submenus[name]->vars[TITLE_VAR]=name;
-  } else 
-    vars=v;
-  vars[SECTION_VAR]=fullsec;
-}
-void menuentry::output(){
-  String treew=config->treewalk();
-  map <String, menuentry *, less<String> >::iterator i;
-
-  for(unsigned int j=0;j<treew.length(); j++){
-    bool children_too=false;
-    switch(treew[j]){
-    case 'c':
-      for(i=submenus.begin(); i!=submenus.end(); i++)
-	if((*i).second->submenus.size())
-	  (*i).second->output();
-      break;
-    case '(':
-      if(submenus.size())
-	if(config->startmenu)
-	  config->startmenu->output(vars);
-      break;
-    case ')':
-      if(submenus.size())
-	if(config->endmenu)
-	  config->endmenu->output(vars);
-      break;
-    case 'M':
-      children_too=true;
-    case 'm':;
-      for(i=submenus.begin(); i!=submenus.end(); i++)
-        if((*i).second->vars[COMMAND_VAR].length())
-          supported->subst((*i).second->vars);
-        else{
-	  if(config->submenutitle)
-	    config->submenutitle->output((*i).second->vars);
-          if(children_too)
-            (*i).second->output();
-        }
-    }
-  }
-}
-
-void menuentry::postprocess(){
-  map <String, menuentry *, less<String> >::iterator i;
-  int index;
-  for(i=submenus.begin(), index=0; 
-      i!=submenus.end(); 
-      i++, index++){
-    (*i).second->vars[PRIVATE_ENTRYINDEX_VAR]=itoString(index);
-    if((*i).second->submenus.size())
-      (*i).second->postprocess();
-    else
-      (*i).second->vars[PRIVATE_ENTRYCOUNT_VAR]="0";
-  }
-  generate_hotkeys();
-  vars[PRIVATE_ENTRYCOUNT_VAR]=itoString(submenus.size());
-}
-char menuentry::hotkeyconv(char h){
-  if (config->hotkeycase)
-    return h;
-  else
-    return tolower(h);
-}
-void menuentry::generate_hotkeys(){
-  unsigned int i,j;
-  list<int>::iterator k, old_k;
-  map <String, menuentry *, less<String> >::iterator subi;
-  map <String, String, less<String> >::iterator l;
-  vector<String> keys;
-  list<int> todo;
-  set<char, less<char> > used_chars;
-  String s;
-  char c;
-
-  if(config->hkexclude)
-    s=config->hkexclude->soutput(vars);
-  for(i=0;i!=s.length();i++)
-    used_chars.insert(hotkeyconv(s[i]));
-
-  for(subi=submenus.begin(), i=0; subi!=submenus.end(); subi++, i++){
-    todo.push_back(i);
-    l=(*subi).second->vars.find(HOTKEY_VAR);
-    if(l!=(*subi).second->vars.end())
-      keys.push_back((*l).second);
-    else
-      keys.push_back('\0');
-    keys[i]+=sort_hotkey((*subi).second->vars[TITLE_VAR]);
-  }
-  j=0;
-  while(todo.size()){
-    for(k=todo.begin();k!=todo.end();){
-      i=*k; 
-      old_k=k++;//k++ here, to be able to todo.erase(old_k) safely.
-      if(j>=keys[i].length()){
-	keys[i]='\0';
-	todo.erase(old_k);  //no hotkey found -- give up on this entry.
-	continue;
-      }
-      c=keys[i][j];
-      if(c){
-	if(used_chars.find(hotkeyconv(c))==used_chars.end()){
-	  todo.erase(old_k); //found a hotkey for this entry.
-	  keys[i]=c;
-	  used_chars.insert(hotkeyconv(c));
-	  continue;
-	}
-	else
-	  keys[i][j]='\0';
-      }
-    }
-    j++;
-  }
-  for(subi=submenus.begin(), i=0; subi!=submenus.end(); subi++, i++){
-    c=keys[i][0];
-    if(c){
-      (*subi).second->vars[HOTKEY_VAR]=c;
-    }
-  }
-}
 /////////////////////////////////////////////////////
 //  Misc
 //
@@ -972,6 +1000,7 @@ map <String, String, less<String> > read_vars(parsestream &i){
   catch(endofline p){};
   return m;
 }
+
 void check_vars(parsestream &i,
 		map <String, String, less<String> > &m){
   vector <String> need;
@@ -993,15 +1022,21 @@ void read_input(parsestream &i){
   try{
     while(1){
       map <String, String, less<String> > m;
-      String sec;
+      StrVec sec_vec;
+      
       m=read_vars(i);
+
+      // check presence of section,title,needs vars. Later we will blindly
+      // assume they are defined.
       check_vars(i,m);
-      sec=m[SECTION_VAR]+String('/');
-      if(m.find(SORT_VAR)!=m.end())
-	sec=sec+m[SORT_VAR]+":";
-      sec=sec+replace(m[TITLE_VAR],'/','_');
+
+      break_slashes(m[SECTION_VAR],sec_vec);
+      //if(m.find(SORT_VAR)!=m.end())
+      //	sec_vec.push_back(m[SORT_VAR]+":"+m[TITLE_VAR]);
+      //else
+      sec_vec.push_back(m[TITLE_VAR]);	  
       if(supported->prec(m[NEEDS_VAR])!=MAXINT)
-	menu.add_entry(sec,config->rootsection(),m);
+	menu.add_entry(sec_vec,m);
       i.skip_line();   //read away the final newline
     }
   }
@@ -1016,15 +1051,15 @@ void includemenus(String o, String i,
   char buf[MAX_BUF];
   char c;
   bool changed=false;
-  ifstream fi(i);
-  ifstream fm(m);
+  ifstream fi(i.c_str());
+  ifstream fm(m.c_str());
   
   if(!fi){
     cerr<<"Cannot open file "<<i<<endl; throw informed_fatal();}
   if(!fm){
     cerr<<"Cannot open file "<<m<<endl; throw informed_fatal();}
 
-  ofstream fo(o);
+  ofstream fo(o.c_str());
 
   if(!fo){
     cerr<<"Cannot open file "<<o<<endl; throw informed_fatal();}
@@ -1089,12 +1124,28 @@ int main(int argc, char **argv){
       supported->debuginfo(cerr);
     }
     if(config->prerun)
-      system(config->prerun->soutput(menu.vars));
+      system((config->prerun->soutput(menu.vars)).c_str());
+    if(config->preruntest){
+      bool r;
+      r=system((config->preruntest->soutput(menu.vars)).c_str());
+      if(r)
+	return r;
+    }
+    if(config->onlyrunasroot)
+      if(getuid())
+	return 0;
+    if(config->onlyrunasuser)
+      if(!getuid())
+	return 0;
     parsestream psscript(cin);
+    menu.vars[TITLE_VAR]=config->mainmenutitle();
+
     read_input(psscript);
-    if(menu.vars[TITLE_VAR]==String(""))
-      menu.vars[TITLE_VAR]=config->mainmenutitle();
-    menu.postprocess();
+    if(config->hint_optimize){ 
+      StrVec v;
+      menu.process_hints(v);
+    };
+    menu.postprocess(1,0, config->rootsection());
     menu.output();
     closegenoutput();
     if(config->rcfile().length()&&config->examplercfile().length())
@@ -1103,7 +1154,7 @@ int main(int argc, char **argv){
 		   "include-menu-defs",
 		   config->prefix()+"/"+config->genmenu->soutput(menu.vars));
     if(config->postrun)
-      system(config->postrun->soutput(menu.vars));
+      system((config->postrun->soutput(menu.vars)).c_str());
     exit(0);
   }
   //yes, I know this _should_ be handled in one
@@ -1115,15 +1166,19 @@ int main(int argc, char **argv){
   catch(endofline p){        p.report(cerr); }
   catch(char_expected p){    p.report(cerr); }
   catch(char_unexpected p){  p.report(cerr); }
+  catch(unexpectedtrailing p){p.report(cerr);}
+  catch(boolean_expected p){ p.report(cerr); }
   catch(ident_expected p){   p.report(cerr); }
   catch(narg_mismatch p){    p.report(cerr); }
   catch(unknown_function p){ p.report(cerr); }
   catch(unknown_ident p){    p.report(cerr); }
   catch(ferror_open p){      p.report(cerr); }
   catch(missing_tag p){      p.report(cerr); }
-  //catch(dir_createerror d){
-  //  cerr<<": Cannot create directory "<<d.name<<endl;
-  //}
+  catch(unknown_compat p){   p.report(cerr); }
+
+  catch(dir_createerror d){
+    cerr<<": Cannot create directory "<<d.name<<endl;
+  }
   catch(informed_fatal){};
 
   cerr<<script<<": Aborting"<<endl;
