@@ -26,6 +26,10 @@
 int debug=0, verbose=0;
 set <String, less<String> > installed_packages;
 set <String, less<String> > menufiles_processed;
+
+extern char **environ;
+
+
 translateinfo *transinfo;
 configinfo     config;
 
@@ -560,16 +564,19 @@ void read_menufilesdir(vector<String> &menudata){
 	    menufiles_processed.insert(name);
 	    name=dirname+name;
 	    r=stat(name.c_str(),&st);
-	    if((!r)&&(S_ISREG(st.st_mode)||S_ISLNK(st.st_mode)))
-	      read_menufile(name,entry->d_name,
-			    menudata);
+	    try {
+	      if((!r)&&(S_ISREG(st.st_mode)||S_ISLNK(st.st_mode)))
+		read_menufile(name,entry->d_name,
+			      menudata);
+	    }
+	    catch(endofline p){
+	      cerr << "error reading " << name << endl;
+	    }
 	  }
-	
       }
     } catch (dir_error_read p){dummy=p.name;}
   }
 }
-
 void run_menumethod(String methodname,
 		    const vector<String> &menudata){
   const char *str;
@@ -602,9 +609,12 @@ void run_menumethod(String methodname,
     // postrun etc) don't work when ran from dpkg. (When run from dpkg,
     // we elsewhere close all filedescriptors. And apparently
     // we need STDOUT open somehow in order to make the pipes work.
+    // (changed by suggestion of Fabian Sturm <f@rtfs.org>, for GNU Hurd,
+    // bug #105674
+
     close(1);
-    dup(fds[0]); 
-    execve(args[0],(char **)args, NULL);
+    open("/dev/null", O_RDWR);
+    execve(args[0],(char **)args, environ);
     exit(1);
   } else {
     //parent:
@@ -714,6 +724,7 @@ int check_dpkglock(){
   int fd;
   struct flock fl;
   char buf[MAX_LINE];
+  int er;
   if(getuid()){
     config.report("update-menus run by user -- cannot determine if dpkg is "
 		  "locking "DPKG_LOCKFILE": assuming there is no lock",
@@ -727,15 +738,16 @@ int check_dpkglock(){
   fl.l_type= F_WRLCK;
   fl.l_whence= SEEK_SET;
   fl.l_start= 0;
-  fl.l_len= 1;
+  fl.l_len= 0;
   if (fcntl(fd,F_SETLK,&fl) == -1) {
+    er=errno;
     close(fd);
-    if (errno == EWOULDBLOCK || errno == EAGAIN || errno == EACCES)
+    if (er == EWOULDBLOCK || er == EAGAIN || er == EACCES)
       return 1;
     cerr<<"update-menus: Encountered an unknown errno (="
-	<<errno<<")."<<endl
+	<<er<<")."<<endl
 	<<"update-menus: Could you please be so kind as to email joostje@debian.org"<<endl
-	<<"update-menus: the errno (="<<errno<<") with a discription of what you did to"<<endl
+	<<"update-menus: the errno (="<<er<<") with a discription of what you did to"<<endl
 	<<"update-menus: trigger this. Thanks very much."<<endl
 	<<"Press enter"<<endl;
     cin.get(buf,sizeof(buf));
@@ -744,7 +756,7 @@ int check_dpkglock(){
   fl.l_type= F_UNLCK;
   fl.l_whence= SEEK_SET;
   fl.l_start= 0;
-  fl.l_len= 1;
+  fl.l_len= 0;
   if (fcntl(fd,F_SETLK,&fl) == -1){
     // `cannot happen'
     cerr<<"update-menus: ?? Just locked the dpkg status database to see if another dpkg"<<endl
@@ -831,7 +843,10 @@ void wait_dpkg(String &stdoutfile){
       }
     }
   } else {
-    create_lock();
+    r=create_lock();
+    if(!r){
+      exit(1);
+    }
     config.report("Dpkg not locking dpkg status area. Good.",
 		  configinfo::report_verbose);
   }
@@ -915,11 +930,11 @@ int main (int, char **argv){
   String stdoutfile;
   struct stat st;
   int r;
-
-  setlocale (LC_ALL, "");
+  
+  setlocale (LC_MESSAGES, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
-
+  
 
   debug=0;
   try{
